%{
#include "pl_lexer.h"
using namespace Prolog;
#define yyterminate() return Prolog::PLParser::token::END
%}

%option c++
%option case-sensitive
%option yyclass="Prolog::PLLexer"
%option full
%option noyywrap
%option nounput
%option batch
%option yylineno
%option prefix="PL_"

%%

[\ \n\t\r]+                                   {}
"%"[^\n]*\n                                   {}
"("                                           {return Prolog::PLParser::token::OPEN;}
")"                                           {return Prolog::PLParser::token::CLOSE;}
","                                           {return Prolog::PLParser::token::COMMA;}
";"                                           {return Prolog::PLParser::token::SEMICOLON;}
"."                                           {return Prolog::PLParser::token::PERIOD;}
":-"                                          {return Prolog::PLParser::token::IF;}
"?-"                                          {return Prolog::PLParser::token::QUERY;}

"["                                           {return Prolog::PLParser::token::OPENBRACKET;}
"]"                                           {return Prolog::PLParser::token::CLOSEBRACKET;}
"|"                                           {return Prolog::PLParser::token::CLOSEBRACKET;}



[a-z][0-9a-zA-Z]*                             {(*yylval).sval = new(std::string)(yytext); return Prolog::PLParser::token::IDENTIFIER;}
[A-Z][0-9a-zA-Z]*                             {(*yylval).sval = new(std::string)(yytext); return Prolog::PLParser::token::VARIABLE;}
[0-9]+                                        {(*yylval).ival = new(int)(atoi(yytext)); return Prolog::PLParser::token::INT;}
[0-9]*\.[0-9]+                                {(*yylval).fval = new(float)(atof(yytext)); return Prolog::PLParser::token::FLOAT;}
.                                             {std::cerr << "lexical error (" << yylineno << "): unexpected character " << yytext << std::endl;}

%%

Prolog::PLLexer::PLLexer(std::istream& is) : PL_FlexLexer(&is)
{
}

int Prolog::PLLexer::yylex(Prolog::PLParser::semantic_type* lval)
{
    yylval = lval;
    return yylex();
}
