

#include "knowledgebase.h"
using namespace std;

namespace Prolog
{
    bool Prove(KnowledgeBase* knowledgebase, Goal* goal, int& VariableCounter)
    {
        bool result = false;
        if(goal->size() <= 0)
            result = true;


        for(Clauses::iterator i = knowledgebase->clauses->begin(); i != knowledgebase->clauses->end() && !result; i++)
        {
            int LocalVariableCounter = VariableCounter;

            map<string,string> GoalVariables;
            Term* goalFront = (*(goal->begin()))->Clone(LocalVariableCounter, GoalVariables);
            map<string,string> ClauseVariables;
            Term* clauseHead = (*i)->Head->Clone(LocalVariableCounter, ClauseVariables);

            Unifier* mgu = Unify(goalFront, clauseHead);

            if(mgu != NULL)           // clause-head matches goal => clause is tried
            {
                // resolve the goal against the clause
                // Res = (clause \ {head}) union (goal \ {fist element})
                Goal* resolvent = new Goal;


                // the clause-head is not added, because it is resolved against the first element of the goal
                for(Terms::iterator t = (*i)->Block->begin(); t != (*i)->Block->end(); t++)
                {
                    Term* tClone = (*t)->Clone(LocalVariableCounter, ClauseVariables);

                    //tClone->ApplyUnifier(mgu);
                    for(Unifier::iterator u = mgu->begin(); u != mgu->end(); u++)
                        Term::ReplaceVariables(&tClone, static_cast<Variable*>(u->first)->Name, u->second);

                    resolvent->push_back(tClone);
                }


                // the first element is skipped, because this one is resolved against the clause-head
                for(Goal::iterator t = goal->begin()+1; t != goal->end(); t++)
                {
                    Term* tClone = (*t)->Clone(LocalVariableCounter, GoalVariables);

                    //tClone->ApplyUnifier(mgu);
                    for(Unifier::iterator u = mgu->begin(); u != mgu->end(); u++)
                        Term::ReplaceVariables(&tClone, static_cast<Variable*>(u->first)->Name, u->second);

                    resolvent->push_back(tClone);
                }


                result = Prove(knowledgebase, resolvent, LocalVariableCounter);

                for(Goal::iterator i = resolvent->begin(); i != resolvent->end(); i++)
                    delete *i;
                delete resolvent;
                delete mgu;
            }

            delete goalFront;
            delete clauseHead;
        }

        return result;
    }

} // namespace Prolog