
#include "knowledgebase.h"
using namespace std;


namespace Prolog
{

    bool Unify(Unifier* remaining, Unifier* mgu)
    {
        while(remaining->size() > 0)
        {
            Term* t1 = remaining->begin()->first;
            Term* t2 = remaining->begin()->second;
            remaining->erase(remaining->begin());

            CompositeTerm* ct1 = dynamic_cast<CompositeTerm*>(t1);
            CompositeTerm* ct2 = dynamic_cast<CompositeTerm*>(t2);
            Variable* vt1 = dynamic_cast<Variable*>(t1);
            Variable* vt2 = dynamic_cast<Variable*>(t2);

            // case 1: both terms are composite
            if(ct1 != NULL && ct2 != NULL)
            {
                size_t s1 = ct1->Children == NULL ? 0 : ct1->Children->size();
                size_t s2 = ct2->Children == NULL ? 0 : ct2->Children->size();

                // Clash-Check
                if(*(ct1->Head) != *(ct2->Head)
                    || s1 != s2)
                    return false;

                // add the children to the set of remaining term-pairs
                if(s1 > 0   /* && s2 > 0 */)
                {
                    Terms::iterator i1 = ct1->Children->begin();
                    Terms::iterator i2 = ct2->Children->begin();
                    for( ; i1 != ct1->Children->end(); i1++, i2++ )
                        remaining->push_back(pair<Term*, Term*>(*i1, *i2));
                }
            }
            else
            {
                // case 2: term 2 is a variable (-> this situation is turned into case 3)
                if(vt2 != NULL)
                {
                    Variable* temp = vt2;
                    vt2 = vt1;
                    vt1 = temp;

                    ct2 = ct1;
                    t2 = t1;
                }

                // case 3: term 1 is a variable
                if(vt1 != NULL)
                {

                    // Occur-Check
                    if(ct2 != NULL)
                        if(ct2->ContainsVariable(vt1->Name))
                            return false;

                    // substitution
                    mgu->push_back(pair<Term*,Term*>(vt1, t2));
                    for(Unifier::iterator i = remaining->begin(); i != remaining->end(); i++)
                    {
                        Term::ReplaceVariables(&(i->first), vt1->Name, t2);
                        Term::ReplaceVariables(&(i->second), vt1->Name, t2);
                    }

                }
            }
        }

        return true;
    }



    Unifier* Unify(Term* t1, Term* t2)
    {
        Unifier* mgu = new Unifier;
        Unifier* remaining = new Unifier;
        remaining->push_back(pair<Term*, Term*>(t1,t2));
        bool success = Unify(remaining, mgu);

        delete remaining;
        if(!success)
        {
            delete mgu;
            mgu = NULL;
        }
        return mgu;
    }


} // namespace Prolog